/*
 * Copyright (c) 2006-2010 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <GPSNovatel/gpx.h>

// print the header
void
print_gpx_header(FILE *out)
{
  (void)fprintf(out,"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
  (void)fprintf(out,"<gpx version=\"1.1\" creator=\"LAAS GPS-genom\"\n");
  (void)fprintf(out,"        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
  (void)fprintf(out,"        xmlns=\"http://www.topografix.com/GPX/1.1\"\n");
  (void)fprintf(out,"        xsi:schemaLocation=\"http://www.topografix.com/GPS/1/1\n");
  (void)fprintf(out,"        http://www.topografix.com/GPX/1/1/gpx.xsd\">\n");
  (void)fprintf(out," <metadata>\n");
  (void)fprintf(out,"  <name>%s</name>\n",progname);
  (void)fprintf(out,"  <author>%s</author>\n", author);
  (void)fprintf(out,"  <copyright>%s</copyright>\n", license);
  (void)fprintf(out," </metadata>\n");
  (void)fflush(out);
}

// trk represents a track - an ordered list of points describing a path. 
void
print_gpx_trk_start (FILE* out)
{
  (void)fprintf(out," <trk>\n");
  (void)fprintf(out,"  <trkseg>\n");
  (void)fflush(out);
}

// print the track end
void
print_gpx_trk_end (FILE* out)
{
  (void)fprintf(out,"  </trkseg>\n");
  (void)fprintf(out," </trk>\n");
  (void)fflush(out);
}

// print the pos element in the track
void
print_fix(struct gps_fix_t *fix, struct tm *time, FILE* out)
{
  (void)fprintf(out,"   <trkpt lat=\"%f\" lon=\"%f\">\n",
               fix->latitude, fix->longitude);
  (void)fprintf(out,"    <ele>%f</ele>\n",
               fix->altitude);
  (void)fprintf(out,"    <time>%04d-%02d-%02dT%02d:%02d:%02dZ</time>\n",
               time->tm_year+1900, time->tm_mon+1, time->tm_mday,
               time->tm_hour, time->tm_min, time->tm_sec);
  
  (void)fprintf (out, "    <fixType>none</fixType>\n");

  (void)fprintf(out,"   </trkpt>\n");
  (void)fflush (out);
}

/*
 * Make new track if the jump in time is above
 * timeout.  Handle jumps both forward and
 * backwards in time.  The clock sometimes jumps
 * backward when gpsd is submitting junk on the
 * dbus.
 */
void
conditionally_log_fix(struct gps_fix_t *gpsfix, FILE* out)
{
  
  // extract integer part
  int_time = floor(gpsfix->time);
  if (int_time != old_int_time) {
    struct tm   time;
    /*
     * Make new track if the jump in time is above
     * timeout.  Handle jumps both forward and
     * backwards in time.  The clock sometimes jumps
     * backward when gpsd is submitting junk on the
     * dbus.
     */
    if (fabs(int_time - old_int_time) > timeout && !first) {
      print_gpx_trk_end(out);
      intrack = false;
    }

    if (!intrack) {
      print_gpx_trk_start(out);
      intrack = true;
      if (first)
        first = false;
    }

    old_int_time = int_time;
    gmtime_r(&(int_time), &time);
    print_fix(gpsfix, &time, out);
  }
  
}

// print the gpx file footer
void
print_gpx_footer (FILE* out)
{
  if (intrack)
    print_gpx_trk_end(out);
  (void)fprintf(out,"</gpx>\n");
  (void)fclose(out);
}
